@@ -9,6 +9,8 @@
 #undef USE_KMEM
 #undef SOLARIS
 
+#undef NO_MASQ
+
 /*
 ** Define this if you want random chars of the set [0-9A-Za-z] instead of
 ** UPREFIXxxxxx
@@ -17,7 +19,7 @@
 
 /*
 ** Include support for the handling of requests for IP Masqueraded
-** connections. (Linux only)
+** connections.
 */
 #undef MASQ_SUPPORT
 
