@@ -15,7 +15,7 @@
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 **
-** $Id: oidentd.h,v 1.6 2000/01/02 00:44:49 odin Exp $
+** $Id: oidentd.h,v 1.10 2000/08/21 22:22:08 odin Exp $
 */
 
 /*
@@ -27,8 +27,8 @@
 
 #define _GNU_SOURCE
 
-#include <stdio.h>
 #include <unistd.h>
+#include <stdio.h>
 #include <stdarg.h>
 #include <syslog.h>
 #include <errno.h>
@@ -63,6 +63,7 @@
 #define FWD			0x8000
 #define PROXY		0x10000
 #define QUIET		0x20000
+#define USEUDB      0x40000
 
 /*
 ** Maximum length of identd replies
@@ -83,9 +84,9 @@
 #define DEFAULT_WTIMEOUT	120
 #define DEFAULT_UMASK		0022
 
-#define FACILITY LOG_DAEMON
-#define PRIORITY LOG_INFO
-#define DPRI LOG_DEBUG
+#define FACILITY	LOG_DAEMON
+#define PRIORITY	LOG_INFO
+#define DPRI		LOG_DEBUG
 
 /*
 ** 0: User is not allowed to spoof identd replies
@@ -103,22 +104,24 @@
 #define EMAIL	"odin@numb.org"
 #define URL		"http://ojnk.sourceforge.net"
 
-void *xmalloc(size_t);
-int get_user(int, int, const struct in_addr *, const struct in_addr *);
-int get_addr(const u_char *const, u_long *);
-ssize_t sockread(int fd, u_char *buf, size_t len);
-
-#ifndef HAVE_DPRINTF
-	int dprintf(int, const char *, ...);
+#ifdef NO_MASQ
+#	undef MASQ_SUPPORT /* not all versions do support it so far */
 #endif
 
-#if defined(__linux__)
-#	if defined(MASQ_SUPPORT)
+#ifdef MASQ_SUPPORT
+#	ifdef __linux__
 		int masq(int, int, int, const struct in_addr *);
-#	endif /* masq */
-#else
-		int k_open(void);
-#endif /* __linux__ */
+#	else /* masq */
+		int masq(int, int, const struct in_addr *, int, const struct in_addr *);
+#	endif
 
+	int find_entry(u_long host, u_char *user, size_t ursz, u_char *os, size_t osz);
+#endif
+
+#ifndef __linux__
+	int k_open(void);
+#endif
+
+int get_user(int lport, int fport, const struct in_addr *laddr, const struct in_addr *faddr);
 
 #endif /* __OIDENTD_H_ */
